:root > * {
    --md-accent-fg-color: #0075ff;
    --md-primary-fg-color: #0075ff;
    --md-primary-fg-color--dark: #0075ff;
}

:root, [data-md-color-scheme=default] {
    --header-bg-color: rgba(0, 75, 255, 0.75);
}

:root, [data-md-color-scheme=slate] {
    --header-bg-color: rgba(0, 117, 255, 0.75);
}

.md-typeset .tabbed-labels:before {
    background: none !important;
    border-bottom: .1rem solid;
    border-bottom-color: var(--md-primary-fg-color);
}

.md-header, .md-tabs {
    background-color: var(--header-bg-color);
    backdrop-filter: blur(4px);
}

.md-header__button.md-logo > img {
    border-radius: 4px;
}

.hljs-comment, .hljs-quote {
    color: #5c6370;
    font-weight: 500;
}

.hljs-formula, .hljs-keyword, .hljs-literal {
    color: #D55FDE
}

.hljs-addition, .hljs-attribute, .hljs-string, .hljs-regexp, .hljs-string {
    color: #2fb170
}

.hljs-attr {
    color: #abb2bf
}

.hljs-type, .hljs-meta {
    color: #E4B962;
}

.hljs-number, .hljs-selector-attr, .hljs-selector-class, .hljs-selector-pseudo, .hljs-template-variable, .hljs-variable {
    color: #d0864a
}

.hljs-bullet, .hljs-link, .hljs-selector-id, .hljs-symbol, .hljs-title, .hljs-class .hljs-title, .hljs-title.class_ {
    color: #5488e8
}

.hljs-built_in, .hljs-deletion, .hljs-name, .hljs-section, .hljs-selector-tag, .hljs-subst {
    color: #2AAFB0
}

.hljs-emphasis {
    font-style: italic
}

.hljs-strong {
    font-weight: 700
}

.hljs-link {
    text-decoration: underline
}

.hljs-sql-text {
    color: var(--md-code-fg-color);
}

.md-typeset .grid.cards > ul > li {
    border-radius: 5px;
}

.md-nav__title .md-nav__button.md-logo img {
    border-radius: 5px;
}

#bulk-overview {
    width: 100%;
    height: 400px;
    border: none;
    border-radius: 10px;
    background-color: white;
}
